### skkserv/ebdic.rb --- rskkserv module for EB dictionary.

## Copyright (C) 2000,2001  YAMASHITA Junji

## Author:	YAMASHITA Junji <ysjj@unixuser.org>
## Version:	1.0

## This file is part of rskkserv.

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License as
## published by the Free Software Foundation; either version 2, or (at
## your option) any later version.

## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.

### Code:

require "eb"

require "skkserv/logger"

module EB
  class Book
    alias dictionary directory
  end
end

# ddskk/skk-vars.el  skk-lookup-option-alist ѿ򻲹ͤˤ
module EPWAgent
  #  => Ф() => Ѵ()

  # KOUJIEN:  
  # »
  # (1) ޤñѴ˴ޤʤ
  # (2) "ۤ⤲ڥۥ" Τ褦
  #	3ʾ夫 2ܰʹߤȤƤ
  #	ܤˤĤƤ(¿ʬ)˾ޤѴФʤ
  # (3) 겾̾ˤбƤʤ
  module KOUJIEN
    # :
    # ""
    #	=> ġۥ", "ڲۥ"
    #	=> "", ""
    # ""
    #	=> "oui ե󥹡"
    #	=> "oui", "ե"
    # "衼ä"
    #	=> "衼åѡEuropa ݥȥ롦á"
    #	=> "Europa", "ݥȥ", "", ""
    # "Ρ٤뤷礦"
    #	=> "Ρ٥롾礦ڡޡۡť䥦"
    #	=> "Ρ٥"
    # "Ȥޤ"
    #	=> "ȥޥ<?>tman "
    #	=> ""
    # "ʤƤäɤơĤ֤꤫"
    #	=> "ʥƥåɡơġ֡ꥫUnited States of America"
    #	=> "United States of America"
    # "路Ȥ"
    #	=> "亮ȥWashington", "亮ȥWashingtonܡ"
    #	=> "Washington", "Washington", ""
    MATCH_REGEXP = /\A([^]+)(([ .0-9<=>?A-Za-z]+) ([-]+)?)?([^]+)?/e

    HEAD_SUBST_REGEXP = /\A/e

    MIDPOINT_EUCJP = "\xa1\xa6"
    HYPHEN_EUCJP = "\xa1\xbe"
    
    GAIJI = "<?>"
    STEM_DELIMITER_EUCJP = MIDPOINT_EUCJP
    WORD_DELIMITER_EUCJP = HYPHEN_EUCJP

    module_function
    def format(kana, condidates)
      result = []
      condidates.each do |e|
	MATCH_REGEXP =~ e

	if $2
	  result.push($3) if $3.index(GAIJI).nil?
	  result.push($4)
	end

	if $5
	  s = $5
	  words = $1.split(WORD_DELIMITER_EUCJP)
	  s.split(MIDPOINT_EUCJP).each do |v|
	    next if v.index(GAIJI)

	    v.sub!(HEAD_SUBST_REGEXP, words[0])
	    result.push(v)
	  end
	end
      end
      result
    end
  end

  # MYPAEDIA-fpw
  module MYPAEDIA
    # :
    # "衼ä"
    #	=> "衼å (Europe)"
    #	=> "衼å", "Europe"
    # "碌"
    #	=> " [碌] ()"
    #	=> ""
    # "Ρ٤뤷礦"
    #	=> "Ρ٥ [Ρ٥뤷礦]"
    #	=> "Ρ٥"
    # "餹"
    #	=> "饹 (Maria Callas)",
    #	   "饹 () [饹]",
    #	   "ɽ [餹] (Į)"
    #	=> "饹", "Maria Callas", "饹", "", "ɽ"
    # ""
    #   => " (/) []"
    #   => "", ""
    # "ˤ"
    #   => "/ [ˤ]"
    #   => "", ""
    MATCH_REGEXP = /\A([^ ]+)( \(([^\)]+)\))?/

    module_function
    def format(kana, condidates)
      result = []
      condidates.each do |e|
	MATCH_REGEXP =~ e
	result.concat($1.split("/")) if $1 != kana
	result.concat($3.split("/")) if $3
      end
      result
    end
  end

  module WDIC
    module_function
    def format(kana, condidates)
      condidates
    end
  end
end

class EBDic
  include EPWAgent

  def initialize(path, subdic)
    Logger::log(Logger::DEBUG, "path %s, subdic %s", path, subdic)

    @book = EB::Book.new
    @book.bind(path)
    @book.subbook_list.each do |n|
      if @book.dictionary(n) == subdic
	@book.subbook = n
	@formatter = EPWAgent.const_get(subdic)
	break
      end
    end
  end

  def search(kana)
    Logger::log(Logger::DEBUG, "search: \"%s\", book: %s", kana, @book)
    [] unless @formatter

    result = []
    begin
      result = @formatter.format(kana,
				 @book.exactsearch2(kana).collect {|a| a[1]})
    rescue RuntimeError
      raise if $!.to_s != "fail searching"
    end

    Logger::log(Logger::DEBUG, "candidates: \"%s\"", result.join(","))
    result
  end

  def self.create_search_agents(serverconfig)
    config = serverconfig::EBDic

    search_agents = []

    # ruby 1.6.2, for get written order in file,
    # need reverse evaluation.
    config.constants.reverse_each do |ms|
      m = config.const_get(ms)
      next if m.type != Module

      search_agents.push(EBDic.new(m.const_get("DICDIR"),
				   m.const_get("SUBDIC")))
    end

    search_agents
  end
end

if __FILE__ == $0
  print EBDic.new("/opt/epwing/koujien", "KOUJIEN").search("").join("/"), "\n"
end

### skkserv/ebdic.rb ends here
